#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_BaseItems.py,v 1.9 2003/04/02 07:57:32 wankyu Exp $
Description

   NeoPortal Base Item Classes
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import ImplicitAcquisitionWrapper, aq_parent, aq_base, aq_inner
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from DateTime.DateTime import DateTime

from Products.NeoPortalLibrary.NP_LoggableObject import NeoPortalLoggableObject
from Products.NeoPortalLibrary.NP_Utils import encryptNeoPortalPassword

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalBaseItem( PropertyManager, SimpleItem, NeoPortalLoggableObject ):
    """
        NeoPortal Base Item Class
    """

    manage_options = (
        PropertyManager.manage_options
        + SimpleItem.manage_options)    
    
    security = ClassSecurityInfo()
    
    security.declarePublic( 'Title' )
    def Title( self ):
        """
            Return title: name changed for compatibility with CMF DublinCore
        """
        
        return self.title_or_id()

    security.declareProtected( NeoPortalModifyPortalContent, 'setTitle' )
    def setTitle( self, title ):
        """
            Set title
        """
        
        self.title = title

    security.declarePublic( 'Creator' )
    def Creator( self ):
        """
            Return creator of this object
        """     

        if hasattr( aq_base( self ), 'creator' ):
            return self.creator

        try:
            if hasattr( aq_base( self ), 'getOwner' ):
                owner = self.getOwner()
                if hasattr( owner, 'getUserName' ):
                    return owner.getUserName()
        except:
            return None
        return None

    security.declarePublic( 'CreatorEmail' )
    def CreatorEmail( self ):
        """
            Return creator email
        """
        creator = self.Creator()
        if creator is None:
            return ''
            
        else:
            try:
                creator = self.portal_membership.getMemberById( creator )
                return creator.getProperty('email')
            except:
                if hasattr( aq_base( self ), 'creator_email'):
                    return self.creator_email
                else:
                    return None
                
    security.declarePublic( 'Authors' )
    def Authors( self ):
        """
            Return authors of this object
        """
        
        if hasattr( self, 'authors' ):
            return self.authors
        
        creators = self.Creator() # hope they fixed this Dublin Core method to return creatorS

        from types import ListType
        if isinstance( creators,ListType ):
            return creators
        else:
            return [creators, ]

    security.declareProtected( NeoPortalModifyPortalContent, 'setCreator' )
    def setCreator( self, creator ):
        """
            Set creator of this object
        """     
        if creator:
            self.creator = creator.strip()
   
    security.declareProtected( NeoPortalModifyPortalContent, 'setCreatorEmail')
    def setCreatorEmail( self, creator_email ):
        """
            Set creator email
        """     
        if creator_email:
            self.creator_email = creator_email.strip()

    security.declarePublic( 'Description' )
    def Description( self ):
        """
            Return description
        """ 
        
        return self.description

    security.declareProtected( NeoPortalModifyPortalContent, 'setDescription')
    def setDescription( self, description ):
        """
            Set description
        """ 
        
        self.description = description


    security.declarePublic( 'CreationDate' )
    def CreationDate( self ):
        """
            Return Creation Date
        """ 
        return self.creation_date and self.creation_date.ISO() or 'Unknown'
    
    security.declarePublic( 'EffectiveDate' )
    def EffectiveDate( self ):
        """
            Return Effective Date
        """ 
        
        ed = getattr( self, 'effective_date', None )
        
        return ed and ed.ISO() or 'None'

    security.declarePublic( 'ExpirationDate' )
    def ExpirationDate( self ):
        """
            Return Expiration Date
        """ 
    
        ed = getattr( self, 'expiration_date', None )
        
        return ed and ed.ISO() or 'None'

    security.declarePublic( 'ModificationDate' )
    def ModificationDate( self ):
        """
            Return Modification Date
        """ 
        
        return self.modified().ISO()

    security.declarePublic( 'Format' )
    def Format( self ):
        """
            Return Renderable Format
        """ 
        
        return self.format

    security.declareProtected( NeoPortalModifyPortalContent, 'setFormat')
    def setFormat( self, format ):
        """
            Set Format
        """ 
        
        self.format = format

    security.declarePublic( 'Language' )
    def Language( self ):
        """
            Return Language
        """
        
        return self.language         

    # override CMF's Language when needed
    getNeoPortalContentLanguage = Language

    security.declareProtected( NeoPortalModifyPortalContent, 'setLanguage')
    def setLanguage( self, language ):
        """
            Set Language
        """
        
        self.language = language
        
    security.declarePublic( 'isObjectExpired' )
    def isObjectExpired( self ):
        """
            Tell if object has expired
        """
        if hasattr( self, 'expiration_date' ):
            if isinstance( self.expiration_date, DateTime ):
                return self.expiration_date.isFuture()
            else: 
                return 0
        else:
            return 0

    security.declarePublic('isObjectEffective')
    def isObjectEffective( self, check_date=None ):
        """
            Tell if object is effective
        """
        if hasattr( self, 'effective_date') and isinstance( self.effective_date, DateTime ):
            return not self.effective_date.isFuture()
        else:
            return 1

    security.declarePublic('getAuthenticatedNeoPortalUser')
    def getAuthenticatedNeoPortalUser( self, REQUEST=None ):
        """
            Return the current user
        """                                     

        try:
            return self.portal_membership.getAuthenticatedMember()
        except:
            return getSecurityManager().getUser()
        
    security.declarePrivate( '_getRemoteIP' )
    def _getRemoteIP( self, REQUEST=None ):
        """
            Return IP of the current user
        """                                     
        if REQUEST is None:
            REQUEST = self.REQUEST

        if REQUEST is None:
            return 'Unknown'
        else:
            try:
                if REQUEST.has_key('REMOTE_HOST') and REQUEST['REMOTE_HOST']:
                    return REQUEST['REMOTE_HOST']
                elif REQUEST.has_key('REMOTE_ADDR') and REQUEST['REMOTE_ADDR']: 
                    return REQUEST['REMOTE_ADDR']
                else:
                    return 'Unknown'
            except:
                    return 'Unknown'

    security.declarePublic( 'isAnonymousNeoPortalUser' )    
    def isAnonymousNeoPortalUser( self ):
        """
            Tell if the current user is anonymous
        """
        
        auth_user = self.getAuthenticatedNeoPortalUser()
        
        if auth_user is None: return 1
        
        if hasattr( auth_user, 'getUserName') and auth_user.getUserName()=='Anonymous User': return 1
        
        return 0
                
    security.declarePublic( 'isNeoPortalContentOwner' )    
    def isNeoPortalContentOwner( self, REQUEST=None ):
        """
            Tell if the user is owner of this object
        """                             
        
        if self.isAnonymousNeoPortalUser():  return 0

        auth_user = self.getAuthenticatedNeoPortalUser()
        
        try:
            if auth_user.has_role( 'Owner', aq_base( self ) ): return 1
            
            if auth_user.getUserName() == self.Creator(): return 1
        except:
            return 0
            
        return 0

    security.declarePublic( 'isNeoPortalManager' )    
    def isNeoPortalManager( self, REQUEST=None ):
        """
            Tell if the user has manager role
        """                             
        
        if self.isAnonymousNeoPortalUser():  return 0

        auth_user = self.getAuthenticatedNeoPortalUser()
        
        if auth_user.has_role( 'Manager', aq_base( self ) ): return 1
        
        return 0                        
                        
    security.declarePrivate( '_setRemoteIP' )
    def _setRemoteIP( self, REQUEST=None ):
        """
            Set Remote IP of the current user
        """                                     
        
        self.remote_ip = self._getRemoteIP(REQUEST)
            
    security.declarePublic( 'getRemoteIP' )
    def getRemoteIP( self ):
        """
            Return the remote IP
        """                                     
        
        if hasattr( self, 'remote_ip' ):
            return self.remote_ip
        else:
            return 'Unknown'

    security.declarePrivate(' getPassword ' )
    def getPassword( self ):
        """
            Return password if set
        """             
        
        if hasattr( self, 'password' ):
            return self.password

        return encryptNeoPortalPassword( self._generatePassword() )

    security.declareProtected( NeoPortalManageProperties, 'setPassword' )
    def setPassword( self, password ):
        """
            Set encrypted password
        """ 
        
        self.password = encryptNeoPortalPassword( password )
        
    security.declareProtected( NeoPortalView, 'checkPassword' )
    def checkPassword( self, password ):
        """
            Validate password
        """ 

        return encryptNeoPortalPassword( password ) == aq_base( self ).getPassword()
        
    security.declarePrivate( '_generatePassword' )
    def _generatePassword( self ):
        """
            Auto-generate password
        """
          
        import string, random
        random.seed()
        chars = string.lowercase[:26] + string.uppercase[:26] + string.digits
        result = []
        for n in range(7):
            result.append( random.choice( chars ) )
            
        return string.join( result, '' )        

    security.declarePublic( 'getNeoPortalUserAcceptLanguage' )
    def getNeoPortalUserAcceptLanguage(self, REQUEST):
        """
            Return current user's accept language header
        """
        
        if not REQUEST.has_key('HTTP_ACCEPT_LANGUAGE'):
            return None

        return REQUEST['HTTP_ACCEPT_LANGUAGE']   

    security.declarePublic( 'checkNeoPortalContentReviewStatus' )
    def checkNeoPortalContentReviewStatus( self ):
        """
            Check content review status
        """        

        try:
            review_state = self.portal_workflow.getInfoFor( self, 'review_state', '')
            if review_state.lower() != 'published' and review_state.lower() != 'public':
                return 0
            else:
                return 1                
        except:
                return 1

    security.declarePublic( 'isContentLanguageAcceptable' )
    def isContentLanguageAcceptable(self, REQUEST=None):
        """
            Tell if the current user can view the current content as it is 
            (useful in figuring out whether to return cached or re-cooked content for i18n)
        """
        if REQUEST is None:
            if hasattr( self, 'REQUEST' ):
                REQUEST = self.REQUEST
            else:
                return 0
        
        accept_languages = self.getNeoPortalUserAcceptLanguage( REQUEST=REQUEST )

        if accept_languages is None: return 0

        languages = [ language.strip().lower() for language in accept_languages.split(',') ]

        for lang in languages:
            # quality value found ex. -> en;q=0.7
            if lang.find( ';' ) > - 1:
                lang = lang.split(';')[0].strip() # only language part needed
            
            if self.getNeoPortalContentLanguage().lower() == lang:
                return 1

        return 0

    def isTemporaryNeoPortalObject( self, obj=None, container=None, id=None ):
        """ 
            Test if the given object is in the making 
        """
        if container is None:
            container = obj.getParentNode()
        if id is None:
            id = obj.getId()
        return not ( id in container.objectIds() )
    

InitializeClass( NeoPortalBaseItem )