#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_CacheableContent.py,v 1.5 2003/01/10 23:24:59 wankyu Exp $
Description

    NeoPortal Cacheable Content caches away rendered content in 'cooked_text' property
    
    NeoPortal Cacheable objects should define their own _np_renderers which is a list of methods used to render raw content
    
    Renderer methods should accept keyword arguments and may as well throw them away if not useful
 
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalCacheableContent( NeoPortalBaseItem ):
    """
        Mix-in that allows content caching: objects using this mix-in must have the '_cookContent' method
    """
    
    neoportal_cacheable_content = 1
        
    _np_renderers = ['_cookContent', ]
    
    security = ClassSecurityInfo()    

    security.declarePublic( 'isContentCacheable' )
    def isContentCacheable( self ):
        """
            Tell if content is cacheable
        """
        
        if not getattr( self, 'neoportal_cacheable_content', 0 ): 
            return 0
       
        
        if hasattr( self, 'np_content_caching_enabled' ):
            return self.np_content_caching_enabled
            
        return 1
    
    security.declarePublic( 'getRawNeoPortalContent' )
    def getRawNeoPortalContent( self ):
        """
            Return raw text
        """     

        # it's a real headache: windows/unix newline mix-up quickfix
        text = getattr( self, 'text', '')
        
        return text.replace('\r\n', '\n')

    security.declareProtected( NeoPortalModifyPortalContent, 'setRawNeoPortalContent' )
    def setRawNeoPortalContent( self , text='' ):
        """
            Set raw text
        """     
        
        self.text = text
        
    security.declarePublic( 'getRenderedNeoPortalContent' )
    def getRenderedNeoPortalContent( self, **kw ):
        """
            Return cooked content
        """     

        cooked_text = getattr( self, 'cooked_text', '')
        
        return cooked_text
        
    security.declarePrivate( '_updateNeoPortalContentDescription' )
    def _updateNeoPortalContentDescription( self, **kw ):
        """
            Update cooked description
        """     

        self.np_cooked_description = self._renderNeoPortalContent( content = self.description )            
        
    security.declarePublic( 'getRenderedNeoPortalContentDescription' )
    def getRenderedNeoPortalContentDescription( self, **kw ):
        """
            Return cooked description 
        """     
        if not hasattr( aq_base( self ), 'np_cooked_description' ):
            self._updateNeoPortalContentDescription()
        
        return self.np_cooked_description

    security.declarePrivate( '_cookContent' )
    def _cookContent( self, content, **kw ):
        """
            Return content rendered according to the internal rules of content rendering
        """
        
        return content

    security.declarePrivate( '_updateNeoPortalContent' )
    def _updateNeoPortalContent( self ):
        """
            Update Content after adding/edit comments
        """

        self.cooked_text = self._renderNeoPortalContent()

        if hasattr(self, 'reindexObject'):
            self.reindexObject()

    security.declarePrivate( '_getNeoPortalContentRenderers' )
    def _getNeoPortalContentRenderers( self ):
        """
            Return defined renderers
        """
        return self._np_renderers
        
    security.declarePrivate( '_renderNeoPortalContent' )
    def _renderNeoPortalContent( self, content=None, renderers=None, **kw ):
        """
            Cook raw content by pushing it thru the chain of defined renderers
        """
        if content is None:
            self.np_log( message="Empty content. Pass: %s" % self.getId(), summary='NP_CacheableContent' , severity='DEBUG')
            content = self.getRawNeoPortalContent()
            
        if not content:
            return content
        
        if renderers is None:
            renderers = self._getNeoPortalContentRenderers()
                    
        my_kw = kw.copy()
        
        for renderer in renderers:
            self.np_log( message="%s (%s) calling content renderer: %s ( %s )" % (self.meta_type, self.getId(), renderer, kw), summary='NP_CacheableContent' , severity='DEBUG')                        
            try:
                renderer = getattr( self, renderer )                       
                content = apply(renderer, ( content, ), kw )
            except Exception, e: 
                self.np_log( message="_renderNeoPortalContent renderer error: %s ( %s )" % ( renderer, e ), summary='NP_CacheableContent' , severity='ERROR')
                pass

        return content       

    security.declarePublic( '__len__' )
    def __len__( self ):
        return len( self.cooked_text ) + len( self.text )
            
InitializeClass( NeoPortalCacheableContent )

