#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_QuotaContainer.py,v 1.3 2003/04/01 16:53:13 wankyu Exp $
Description
    
    NeoPortal Quota Container is a mix-in for folderish objects that can limit the total size and number of subobjects

    NeoPortal Quota Container should be the first base class to inherit
    
"""

from Globals import InitializeClass
from Acquisition import ImplicitAcquisitionWrapper, aq_parent, aq_base, aq_inner, aq_chain
from AccessControl import ClassSecurityInfo
from OFS.ObjectManager import ObjectManager

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalQuotaContainer: 
    """ 
        NeoPortal Quota Container
    """

    neoportal_quota_container = 1
    
    neoportal_no_quota_objects = []     # meta_types that should be excluded
    
    _np_quota_setOb = ObjectManager._setOb
    _np_quota_delOb = ObjectManager._delOb
    
    security = ClassSecurityInfo()

    security.declarePublic( 'isNeoPortalContainerQuotaEnabled' )
    def isNeoPortalContainerQuotaEnabled( self ):
        """
            Tell if Quota is enabled
        """
        
        if hasattr( aq_base( self ), 'neoportal_quota_enabled' ): return self.neoportal_quota_enabled

        return 0

    security.declarePublic( 'getNeoPortalQuota' )
    def getNeoPortalContainerQuota( self ):
        """
            Return Quota
        """
        
        if not hasattr( aq_base( self ), '_neoportal_container_size_quota' ):
            self.setNeoPortalContainerQuota()
        
        return ( self._neoportal_container_size_quota, self._neoportal_container_number_quota )

    security.declarePublic( 'getNeoPortalQuotaContainerInfo' )
    def getNeoPortalQuotaContainerInfo( self ):
        """
            Return Container Info
        """
        
        if not hasattr( aq_base( self ), '_neoportal_objects_size' ):
            self._neoportal_objects_size = 0
        if not hasattr( aq_base( self ), '_neoportal_objects_count' ):
            self._neoportal_objects_count = 0
                    
        return ( self._neoportal_objects_size, self._neoportal_objects_count )        

    security.declareProtected( NeoPortalModifyPortalContent, 'setNeoPortalQuota' )
    def setNeoPortalContainerQuota( self, size_quota=0, number_quota=0 ):
        """
            Set Quota: 
            
                size_quota - total size of subobjects
                
                number_quota - total num of subobjects
                
                0 means no quota
        """
        
        self._neoportal_container_size_quota = size_quota   # in bytes
        self._neoportal_container_number_quota = number_quota

    security.declarePrivate( '_checkNeoPortalContainerQuota' )        
    def _checkNeoPortalContainerQuota( self, object ):
        """
            Raise exception if setting object would exceed quota
        """    
        
        
        obj_size = 0
        if hasattr( aq_base( object ), 'get_size'):
            obj_size = object.get_size()
            self.np_log( message="Checking size of %s: %d" % ( object.getId(), obj_size ), summary='NP_QuotaContainer', severity='DEBUG' )
            
        for container in aq_chain( self ):
            if hasattr( container, 'isNeoPortalContainerQuotaEnabled' ) and container.isNeoPortalContainerQuotaEnabled():        
                size, number = container.getNeoPortalQuotaContainerInfo()
                
                self.np_log( message="Checking quota container info(%s): %d byte(s), %d object(s)" % ( container.getId(), size, number ), summary='NP_QuotaContainer', severity='DEBUG' )
                
                size_q, num_q = container.getNeoPortalContainerQuota()
                
                self.np_log( message="Checking quota info(%s): %d byte(s), %d object(s)" % ( container.getId(), size_q, num_q ), summary='NP_QuotaContainer', severity='DEBUG' )
                
                if ( size_q and ( size + obj_size ) > size_q ) or ( num_q and ( number + 1 ) > num_q ):
                    message = self.getNeoPortalMessage( 
                                message="Quota Exceeded: <em>%s</em> allows <b>%d</b> object(s) in number and <b>%d</b> kbytes in size. \
                                         You already have <b>%d</b> objects totalling <b>%d</b> kbytes in size!"
                                        , tup=( self.title_or_id(), num_q, size_q, number, size ) )
                    
                    raise 'NeoPortalError', message

    security.declareProtected( NeoPortalModifyPortalContent, 'recalculateNeoPortalContainerQuota' )        
    def recalculateNeoPortalContainerQuota( self ):        
        """
            Recalculate Quota
        """
        self._recalculateNeoPortalContainerQuota()
            
    security.declarePrivate( '_recalculateNeoPortalContainerQuota' )        
    def _recalculateNeoPortalContainerQuota( self ):        
        """
            Recalculate Quota
        """

        obj_sizes = [ obj.get_size() for id, obj in self.ZopeFind( self, search_sub=1 ) if hasattr( aq_base( obj ), 'get_size') and getattr( obj, 'meta_type', None ) not in self.neoportal_no_quota_objects ]
        if obj_sizes:
            self._neoportal_objects_size = int( reduce( lambda x,y: x+y, obj_sizes ) )
            self._neoportal_objects_count = len( obj_sizes )
        else:
            self._neoportal_objects_size = self._neoportal_objects_count = 0

    security.declarePrivate( '_setOb' )
    def _setOb( self, id, object ):
        """
            Check Quota first
        """
        
        self._checkNeoPortalContainerQuota( object )

        result = self._np_quota_setOb( id, object )

        # no exception? recalcuate quota
        
        self._recalculateNeoPortalContainerQuota()
        
        return result

    security.declarePrivate( '_delOb' )
    def _delOb( self, id ):
        """
            Recalculate Quota first
        """
        object = getattr( self, id )
        
        result = self._np_quota_delOb( id )
                
        self._recalculateNeoPortalContainerQuota()
        
        return result
        
InitializeClass( NeoPortalQuotaContainer )